@@ -2,15 +2,13 @@
 #' List Deployed Applications
 #'
 #' List all applications currently deployed for a given account.
-#' @param account Account name. If a single account is registered on the system
-#'   then this parameter can be omitted.
-#' @param server Server name. Required only if you use the same account name on
-#'   multiple servers.
+#' @inheritParams deployApp
 #' @return
 #' Returns a data frame with the following columns:
 #' \tabular{ll}{
 #' `id`         \tab Application unique id \cr
 #' `name`       \tab Name of application \cr
+#' `title`       \tab Application title \cr
 #' `url`        \tab URL where application can be accessed \cr
 #'
 #' `status`     \tab Current status of application. Valid values are `pending`,
@@ -39,19 +37,21 @@
 applications <- function(account = NULL, server = NULL) {
 
   # resolve account and create connect client
-  accountDetails <- accountInfo(resolveAccount(account, server), server)
+  accountDetails <- accountInfo(account, server)
   serverDetails <- serverInfo(accountDetails$server)
   client <- clientForAccount(accountDetails)
 
-  isConnect <- isConnectInfo(accountInfo = accountDetails)
+  isConnect <- isConnectServer(accountDetails$server)
 
   # retrieve applications
   apps <- client$listApplications(accountDetails$accountId)
 
+  # extract the subset of fields we're interested in
   keep <- if (isConnect) {
     c(
       "id",
       "name",
+      "title",
       "url",
       "build_status",
       "created_time",
@@ -69,7 +69,6 @@ applications <- function(account = NULL, server = NULL) {
       "deployment"
     )
   }
-  # extract the subset of fields we're interested in
   res <- lapply(apps, `[`, keep)
 
   res <- if (isConnect) {
@@ -77,6 +76,7 @@ applications <- function(account = NULL, server = NULL) {
       # set size and instance to NA since Connect doesn't return this info
       x$size <- NA
       x$instances <- NA
+      x$title <- x$title %||% NA_character_
       x
     })
   } else {
@@ -90,6 +90,7 @@ applications <- function(account = NULL, server = NULL) {
         x$instances <- NA
       x$deployment <- NULL
       x$guid <- NA
+      x$title <- NA_character_
       x
     })
   }
@@ -107,10 +108,8 @@ applications <- function(account = NULL, server = NULL) {
   })
 
   # convert to data frame
-  rbindWithoutFactors <- function(...) {
-    rbind.data.frame(..., stringsAsFactors = FALSE)
-  }
-  res <- do.call(rbindWithoutFactors, res)
+  res <- lapply(res, as.data.frame, stringsAsFactors = FALSE)
+  res <- do.call("rbind", res)
 
   # Ensure the Connect and ShinyApps.io data frames have same column names
   idx <- match("last_deployed_time", names(res))
@@ -133,21 +132,32 @@ resolveApplication <- function(accountDetails, appName) {
   stopWithApplicationNotFound(appName)
 }
 
+getApplication <- function(account, server, appId) {
+  accountDetails <- accountInfo(account, server)
+  client <- clientForAccount(accountDetails)
+
+  withCallingHandlers(
+    client$getApplication(appId, "unknown"),
+    rsconnect_http_404 = function(err) {
+      cli::cli_abort("Can't find app with id {.str {appId}}", parent = err)
+    }
+  )
+}
+
 stopWithApplicationNotFound <- function(appName) {
   stop(paste("No application named '", appName, "' is currently deployed",
              sep = ""), call. = FALSE)
 }
 
-applicationTask <- function(taskDef, appName, account, server, quiet) {
+applicationTask <- function(taskDef, appName, accountDetails, quiet) {
+
+  # resolve target account and application
+  application <- resolveApplication(accountDetails, appName)
 
   # get status function and display initial status
   displayStatus <- displayStatus(quiet)
   displayStatus(paste(taskDef$beginStatus, "...\n", sep = ""))
 
-  # resolve target account and application
-  accountDetails <- accountInfo(resolveAccount(account, server), server)
-  application <- resolveApplication(accountDetails, appName)
-
   # perform the action
   client <- clientForAccount(accountDetails)
   task <- taskDef$action(client, application)
@@ -160,17 +170,17 @@ applicationTask <- function(taskDef, appName, account, server, quiet) {
 # streams application logs from ShinyApps
 streamApplicationLogs <- function(authInfo, applicationId, entries, skip) {
   # build the URL
-  url <- paste0(shinyappsServerInfo()$url, "/applications/", applicationId,
+  url <- paste0(serverInfo("shinyapps.io")$url, "/applications/", applicationId,
                 "/logs?", "count=", entries, "&tail=1")
   parsed <- parseHttpUrl(url)
 
   # create the curl handle and perform the minimum necessary to create an
   # authenticated request. we ignore the rsconnect.http option here because only
   # curl supports the kind of streaming connection that we need.
-  handle <- createCurlHandle(NULL, NULL)
-  curl::handle_setopt(handle, customrequest = "GET")
+  handle <- createCurlHandle("GET")
   curl::handle_setheaders(handle,
-    .list = signatureHeaders(authInfo, "GET", parsed$path, NULL))
+    .list = signatureHeaders(authInfo, "GET", parsed$path, NULL)
+  )
 
   # begin the stream
   curl::curl_fetch_stream(url = url,
@@ -208,10 +218,15 @@ showLogs <- function(appPath = getwd(), appFile = NULL, appName = NULL,
                      account = NULL, server = NULL, entries = 50, streaming = FALSE) {
 
   # determine the log target and target account info
-  target <- deploymentTarget(appPath, appName, NULL, NULL, account, server)
-  accountDetails <- accountInfo(target$account, target$server)
-  client <- lucidClientForAccount(accountDetails)
-  application <- getAppByName(client, accountDetails, target$appName)
+  deployment <- findDeployment(
+    appPath = appPath,
+    appName = appName,
+    server = server,
+    account = account
+  )
+  accountDetails <- accountInfo(deployment$account, deployment$server)
+  client <- clientForAccount(accountDetails)
+  application <- getAppByName(client, accountDetails, deployment$name)
   if (is.null(application))
     stop("No application found. Specify the application's directory, name, ",
          "and/or associated account.")
@@ -244,79 +259,53 @@ showLogs <- function(appPath = getwd(), appFile = NULL, appName = NULL,
   }
 }
 
-#' Sync Application Metadata
+#' Update deployment records
 #'
-#' Update the metadata for requested application across all deployments
+#' Update the deployment records for applications published to Posit Connect.
+#' This updates application title and URL, and deletes records for deployments
+#' where the application has been deleted on the server.
 #'
 #' @param appPath The path to the directory or file that was deployed.
-#'
-#' @note This function does not update metadata for Shiny and rpubs apps
-#'
 #' @export
-syncAppMetadata <- function(appPath) {
-  if (is.null(appPath) || !file.exists(appPath)) {
-    stop("appPath is null or does not exist")
-  }
+syncAppMetadata <- function(appPath = ".") {
+  check_directory(appPath)
 
-  # get all of the currently known deployments
   deploys <- deployments(appPath)
+  for (i in seq_len(nrow(deploys))) {
+    curDeploy <- deploys[i, ]
 
-  syncWindow <- getOption("rsconnect.metadata.sync.hours", 24) * 3600
-  now <- as.numeric(Sys.time())
-
-  for (i in 1:nrow(deploys)) {
-    lastSyncTime <- deploys[i, "lastSyncTime"]
-
-    # for legacy dcf files that don't have sync time saved yet
-    if (is.null(lastSyncTime) || is.na(lastSyncTime))
-      lastSyncTime <- deploys[i, "when"]
-
-    # don't sync if within the configured time window
-    if (as.numeric(lastSyncTime) + syncWindow > now) {
+    # don't sync if published to RPubs
+    if (isRPubs(curDeploy$server)) {
       next
     }
 
-    # don't sync non-connect apps
-    if (!isConnectInfo(server = deploys[i, "hostUrl"])) {
-      next
-    }
+    account <- accountInfo(curDeploy$account, curDeploy$server)
+    client <- clientForAccount(account)
 
-    account <- rsconnect::accountInfo(deploys[i, "account"],
-                                      server = deploys[i, "server"])
-
-    connect <- clientForAccount(account)
-
-    application <- NULL
-
-    # if the app does not exist, delete the file
-    tryCatch({
-      application <- connect$getApplication(deploys[i, "appId"])
-    }, error = function(c) {
-      message(paste("appId", deploys[i, "appId"], "no longer exists, deleting config file"))
-      file.remove(deploys[i, "deploymentFile"])
-      stop("Removed config file ", deploys[i, "deploymentFile"])
-    })
-
-    record <- deploymentRecord(
-      name = deploys[i, "name"],
-      title = application$title,
-      username = deploys[i, "username"],
-      account = deploys[i, "account"],
-      server = deploys[i, "server"],
-      hostUrl = deploys[i, "hostUrl"],
-      appId = deploys[i, "appId"],
-      bundleId = deploys[i, "bundleId"],
-      url = application$url,
-      when = deploys[i, "when"],
-      lastSyncTime = now,
-      metadata = list(
-        asMultiple = deploys[i, "asMultiple"],
-        asStatic = deploys[i, "asStatic"],
-        vanity_url = application$vanity_url
-      )
+    application <- tryCatch(
+      client$getApplication(curDeploy$appId),
+      rsconnect_http_404 = function(c) {
+        # if the app has been deleted, delete the deployment record
+        file.remove(curDeploy$deploymentFile)
+        cli::cli_inform("Deleting deployment record for deleted app {curDeploy$appId}.")
+        NULL
+      }
     )
+    if (is.null(application)) {
+      next
+    }
 
     # update the record and save out a new config file
-    writeDeploymentRecord(record, deploys[i, "deploymentFile"])
+    path <- curDeploy$deploymentFile
+    curDeploy$deploymentFile <- NULL # added on read
+
+    # remove old fields
+    curDeploy$when <- NULL
+    curDeploy$lastSyncTime <- NULL
+
+    curDeploy$title <- application$title
+    curDeploy$url <- application$url
+
+    writeDeploymentRecord(curDeploy, path)
   }
 }
